@@ -1,10 +1,17 @@
 import importlib
 import logging
+import mimetypes
 import os
+import re
+import subprocess
+import tempfile
 import urllib.parse
 from collections import OrderedDict
+from contextlib import contextmanager
+from datetime import timedelta
 from logging import getLogger
-from typing import Any, Literal
+from pathlib import Path
+from typing import Any, Literal, TypedDict
 from urllib.parse import urlparse
 
 import jwt
@@ -12,8 +19,10 @@ import pytz
 import requests
 from django.conf import settings
 from django.core.cache import cache
+from django.core.files.uploadedfile import InMemoryUploadedFile, TemporaryUploadedFile
 from django.utils import timezone, translation
 from faker import Faker
+from rest_framework.exceptions import ValidationError
 
 from .enums import LanguageCodeType
 from .translator import Translator
@@ -28,11 +37,11 @@ LEARNGUAL_SERVICE_API_KEY = getattr(
 
 
 LOCAL_URLS = {
-    "iam": "http://host.docker.internal:8000",
-    "learn": "http://host.docker.internal:8001",
-    "payment": "http://host.docker.internal:8002",
-    "notify": "http://host.docker.internal:8003",
-    "media": "http://host.docker.internal:8004",
+    "iam": "http://django-iam:8000",
+    "learn": "http://django-learn:8000",
+    "payment": "http://django-pay:8000",
+    "notify": "http://django-notify:8000",
+    "media": "http://django-media:8000",
 }
 
 
@@ -755,3 +764,180 @@ def get_language_code(language: str) -> str:
         language = "EN"
     language = language.strip().upper()
     return languages.get(language, language)
+
+
+@contextmanager
+def with_language(lang):
+    """
+    A context manager that activates the provided language for the duration
+    of the context block and reverts it to the previous language afterward.
+    """
+    # Get the current language
+    current_language = translation.get_language()
+    # Activate the new language
+    if lang:
+        translation.activate(get_language_code(lang))
+    try:
+        yield
+    finally:
+        # Revert to the previous language
+        translation.activate(current_language)
+
+
+class MediaFileInfo(TypedDict, total=False):
+    mime_type: str | None  # MIME type (optional)
+    duration: str  # Duration as a string (e.g., "00:00:00.000")
+    duration_obj: timedelta  # Duration as a timedelta object
+    video_codec: str | None  # Video codec (optional, only for video files)
+    resolution: str | None  # Resolution (optional, only for video files)
+    frame_rate: str | None  # Frame rate (optional, only for video files)
+    audio_codec: str | None  # Audio codec (optional, for audio/video files)
+    bitrate: str | None  # Bitrate (optional, for audio/video files)
+    sample_rate: str | None  # Sample rate (optional, for audio files)
+    channels: str | None  # Number of audio channels (optional, for audio files)
+
+
+def get_media_file_info(file_path) -> MediaFileInfo:
+    # Get the MIME type of the file
+    mime_type, _ = mimetypes.guess_type(file_path)
+    if not Path(file_path).exists():
+        logger.error(f"{file_path} does not exists.")
+        raise ValidationError("File is not valid")
+    # Run ffmpeg command to extract information about the file
+    command = ["ffmpeg", "-i", file_path]
+    result = subprocess.run(command, capture_output=True, text=True)
+
+    # Extract stderr output (because ffmpeg prints info to stderr)
+    output = result.stderr
+
+    # Initialize the dictionary to store extracted info
+    file_info = {
+        "mime_type": mime_type,
+        "duration": "00:00:00.000",
+        "duration_obj": timedelta(seconds=0),  # Default timedelta
+    }
+    # Extracting duration as a string
+    duration_match = re.search(r"Duration:\s+(\d+):(\d+):(\d+\.\d+)", output)
+    if duration_match:
+        hours, minutes, seconds = map(float, duration_match.groups())
+        # Store duration as string
+        file_info["duration"] = f"{int(hours):02}:{int(minutes):02}:{seconds:06.3f}"
+        # Convert duration to timedelta
+        file_info["duration_obj"] = timedelta(
+            hours=hours, minutes=minutes, seconds=seconds
+        )
+
+    # Extracting video-specific info if it’s a video file
+    if mime_type and mime_type.startswith("video"):
+        video_stream_match = re.search(r"Stream.*Video:\s+(\w+)", output)
+        if video_stream_match:
+            file_info["video_codec"] = video_stream_match.group(1)
+
+        # Extracting resolution
+        resolution_match = re.search(r"(\d+x\d+)", output)
+        if resolution_match:
+            file_info["resolution"] = resolution_match.group(1)
+
+        # Extracting frame rate
+        frame_rate_match = re.search(r"(\d+\.\d+)\s+fps", output)
+        if frame_rate_match:
+            file_info["frame_rate"] = frame_rate_match.group(1)
+
+    # Extracting audio-specific info if it’s an audio or video file
+    if mime_type and (mime_type.startswith("audio") or mime_type.startswith("video")):
+        audio_codec_match = re.search(r"Stream.*Audio:\s+(\w+)", output)
+        if audio_codec_match:
+            file_info["audio_codec"] = audio_codec_match.group(1)
+
+        # Extracting bitrate
+        bitrate_match = re.search(r"bitrate:\s+(\d+\s+\w+)", output)
+        if bitrate_match:
+            file_info["bitrate"] = bitrate_match.group(1)
+
+        # Extracting sample rate (for audio)
+        sample_rate_match = re.search(r"(\d+)\s+Hz", output)
+        if sample_rate_match:
+            file_info["sample_rate"] = sample_rate_match.group(1)
+
+        # Extracting channel information (for audio)
+        channels_match = re.search(r",\s*(\d+)\s+channels", output)
+        if channels_match:
+            file_info["channels"] = channels_match.group(1)
+
+    return file_info
+
+
+def handle_uploaded_file(
+    file: TemporaryUploadedFile | InMemoryUploadedFile,
+) -> MediaFileInfo:
+    """
+    This function takes either a TemporaryUploadedFile or InMemoryUploadedFile,
+    retrieves the file path, and calls get_media_file_info to return metadata.
+    """
+    media_info = {}
+    # Check if the file is a TemporaryUploadedFile (it has a physical path)
+    if isinstance(file, TemporaryUploadedFile):
+        # File path is already available
+        file_path = file.temporary_file_path()
+        media_info = get_media_file_info(file_path)
+
+    # Check if the file is an InMemoryUploadedFile (we need to write it to disk temporarily)
+    elif isinstance(file, InMemoryUploadedFile):
+        # Create a temporary directory using tempfile
+        with tempfile.TemporaryDirectory() as temp_dir:
+            # Create a temporary file path within the temporary directory
+            temp_file_path = os.path.join(temp_dir, file.name)
+
+            # Write the InMemoryUploadedFile to the temporary file
+            with open(temp_file_path, "wb") as temp_file:
+                for chunk in file.chunks():
+                    temp_file.write(chunk)
+
+            # File path is now available
+            file_path = temp_file_path
+            media_info = get_media_file_info(file_path)
+
+    else:
+        # Raise a validation error if the file type is unsupported
+        raise ValidationError(
+            "Unsupported file type. Please provide a TemporaryUploadedFile or InMemoryUploadedFile."
+        )
+
+    # Now call get_media_file_info with the file path
+
+    return media_info
+
+
+def get_frontend_base_url(
+    link_type: Literal["main", "demo"] = "main",
+    path: str = "",
+    service: Literal["payment", "iam", "learn", "notify", "media"] = "iam",
+) -> str:
+    """Function to construct the frontend url
+
+    Args:
+        link_type (Literal[&quot;main&quot;,&quot;demo&quot;], optional): _description_. Defaults to "main".
+        path (str, optional): _description_. Defaults to "".
+
+    Returns:
+        str: _description_
+    """
+    base_url = get_base_url(service=service)
+    enviroment = "local"
+    if base_url.startswith(
+        "https://dev-117782726-api.learngual.com"
+    ) or base_url.startswith("http://dev-117782726-api.learngual.com"):
+        enviroment = "staging"
+    elif base_url.startswith("https://api.learngual.com") or base_url.startswith(
+        "http://api.learngual.com"
+    ):
+        enviroment = "main"
+    match (enviroment, link_type):
+        case ("staging", "main"):
+            return f"https://559staging.learngual.com/{path.lstrip('/')}"
+        case ("main", "main"):
+            return f"https://learngual.com/{path.lstrip('/')}"
+        case ("local", "main"):
+            return f"{base_url}/{path.lstrip('/')}"
+        case _:
+            return f"https://learngual.com/{path.lstrip('/')}"
